import click
import logging
import configparser
import sys, getopt
import warnings
import datetime
#from multipledispatch import dispatch  # type: ignore

warnings.filterwarnings("ignore", ".*is an invalid version and will not be supported in a future release.*")

from openfisca_france_data.erfs_fpr.input_data_builder import (
    step_01_preprocessing as preprocessing,
    # step_02_imputation_loyer as imputation_loyer,
    step_03_variables_individuelles as variables_individuelles,
    step_04_famille as famille,
    step_05_final as final,
    )

log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)

# BCO : Il ne faut pas envoyer les logs dans ""../logs" car c'est un dossier qui n'existe pas.
# /tmp à le mérite d'exister sur OSX et Linux mais pas sous Windows. Le mieux est de ne pas utiliser de fichier,
# et de rediriger la sortie console vers un fichier quand on en a besoin.
fileHandler = logging.FileHandler("/tmp/build_erfs_fpr_{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")))
fileHandler.setLevel(logging.DEBUG)
log.addHandler(fileHandler)

consoleHandler = logging.StreamHandler()
consoleHandler.setLevel(logging.INFO)
log.addHandler(consoleHandler)


#@dispatch(int)
def build(year: int, export_flattened_df_filepath: str = None) -> None:
    """
    Ici on va nettoyer et formatter les donnés ERFS-FPR, pour les rendre OpenFisca-like
    """

    # Step 01 : la magie de ce qui nous intéresse : le formattage OpenFisca
    #
    # - Formattage des différentes variables
    # - On merge les tables individus / menages
    #
    # Note : c'est ici où on objectivise les hypothèses, step 1
    log.info('\n [[[ Year {} - Step 1 / 5 ]]] \n'.format(year))
    preprocessing.build_merged_dataframes(year = year)

    # Step 02 : Si on veut calculer les allocations logement, il faut faire le matching avec une autre enquête (ENL)
    #
    # openfisca_survey_collection = SurveyCollection(name = 'openfisca')
    # stata_directory = openfisca_survey_collection.config.get('data', 'stata_directory')
    # stata_file = os.path.join(stata_directory, 'log_men_ERFS.dta')
    # imputation_loyer.merge_imputation_loyer(stata_file = stata_file, year = year)
    log.info('\n [[[ Year {} - Step 2 / 5 SKIPPED ]]] \n'.format(year))

    # Step 03 : on commence par les variables indivuelles
    log.info('\n [[[ Year {} - Step 3 / 5 ]]] \n'.format(year))
    variables_individuelles.build_variables_individuelles(year = year)

    # Step 04 : ici on va constituer foyer et famille à partir d'invididu et ménage
    #
    # - On fait individu/ménage pour pouvoir faire des familles (foyers sociaux)
    # - On va faire des suppositions pour faire les familles
    # - On va faire les foyers fiscaux à partir des familles
    # - On va faire de suppositions pour faire les foyers fiscaux
    log.info('\n [[[ Year {} - Step 4 / 5 ]]] \n'.format(year))
    famille.build_famille(year = year)

    # Affreux ! On injectait tout dans un même DataFrame !!!
    # C'est très moche !
    #
    # On crée une df par entité par période.
    # Elles sont stockées dans un fichier h5
    log.info('\n [[[ Year {} - Step 5 / 5 ]]] \n'.format(year))
    final.create_input_data_frame(year = year, export_flattened_df_filepath = export_flattened_df_filepath)


@click.command()
@click.option('-y', '--year', default = 2017, help = "ERFS-FPR year", show_default = True,
    type = int, required = True)
@click.option('-f', '--file', 'export_flattened_df_filepath', default = None,
    help = 'flattened dataframe filepath', show_default = True)
@click.option('-c', '--configfile', default = None,
    help = 'raw_data.ini path to read years to process.', show_default = True)
@click.option('-l', '--log', 'lg', default = "info",
    help = 'level of detail for log output.', show_default = True)
def main(year = 2017, export_flattened_df_filepath = None, configfile = None, lg = "info"):
    import time
    start = time.time()

    catch_errors = False

    # get level of logging
    if lg == "info":
        lgi = logging.INFO
    elif lg == "warn":
        lgi = logging.WARNING
    elif lg == "debug":
        lgi = logging.DEBUG

    logging.basicConfig(stream = sys.stdout, # filename = 'build_erfs_fpr.log', level = lgi,
        format='%(asctime)s - %(name)-12s: %(levelname)s %(module)s - %(funcName)s: %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S')

    log.info("Starting build-erfs-fpr [log: {}]".format(lg))

    # determine which years are to be analyzed, from file if available, else parameter
    if configfile is not None:
        log.warning("Reading years to process from {configfile}")
        years = []

        try:
            config = configparser.ConfigParser()
            config.read(configfile)
            for key in config['erfs_fpr']:
                if key.isnumeric():
                    years.append(int(key))
                    # log.info(f"Adding year {int(key)}")
        except KeyError:
            years = [year]
            log.warning(f"Key 'erfs_fpr' not found in {configfile}, switchin to default {years}")

        if len(years) > 1:
            log.info('Configured multiple years: [{}]'.format(';'.join([str(y) for y in years])))
        else:
            log.info('Configured single year: [{}]'.format(years))

        for year in years:
            log.info('Starting with year {}'.format(year))
            if catch_errors:
                try:
                    build(year = year, export_flattened_df_filepath = export_flattened_df_filepath)
                except Exception as e:
                    log.warning(" == BUILD HAS FAILED FOR YEAR {} == ".format(year))
                    log.warning("Error message:\n{}\nEND OF ERROR MESSAGE\n\n".format(str(e)))
            else:
                build(year = year, export_flattened_df_filepath = export_flattened_df_filepath)

    else:
        log.info('Configured single year: [{}]'.format(year))
        build(year = year, export_flattened_df_filepath = export_flattened_df_filepath)

    # TODO: create_enfants_a_naitre(year = year)
    log.info("\n\n ==> Script finished after {} seconds.".format(round(time.time() - start)))


if __name__ == '__main__':
    main()



import pytest


from openfisca_france import FranceTaxBenefitSystem
from openfisca_france_data.common import get_baremes_salarie


tax_benefit_system = FranceTaxBenefitSystem()
parameters = tax_benefit_system.parameters


target_by_period_by_categorie_salarie = {
    'prive_non_cadre': {
        2016: [
            'agff',
            'arrco',
            'asf',
            'chomage',
            'maladie',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        2017: [
            'agff',
            'arrco',
            'asf',
            'chomage',
            'maladie',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        2018: [
            'agff',
            'arrco',
            'asf',
            'chomage',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        2019: [
            'agirc_arrco',
            'asf',
            'ceg',
            'cet2019',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        },
    'prive_cadre': {
        2016: [
            'agff',
            'agirc',
            'apec',
            'arrco',
            'asf',
            'cet',
            'chomage',
            'maladie',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        2017: [
            'agff',
            'arrco',
            'agirc',
            'apec',
            'asf',
            'cet',
            'maladie',
            'chomage',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        2018: [
            'agff',
            'arrco',
            'agirc',
            'apec',
            'asf',
            'cet',
            'chomage',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        2019: [
            'agirc_arrco',
            'asf',
            'apec',
            'ceg',
            'cet2019',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        },
    'public_non_titulaire': {
        2016: [
            'excep_solidarite',
            'ircantec',
            'maladie',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        2017: [
            'excep_solidarite',
            'ircantec',
            'maladie',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        2018: [
            'ircantec',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        2019: [
            'ircantec',
            'vieillesse_deplafonnee',
            'vieillesse_plafonnee',
            ],
        },
    "public_titulaire_etat": {
        2016: [
            "excep_solidarite",
            "pension",
            "rafp",
            ],
        2017: [
            "excep_solidarite",
            "pension",
            "rafp",
            ],
        2018: [
            "pension",
            "rafp",
            ],
        2019: [
            "pension",
            "rafp",
            ],
        },
    "public_titulaire_hospitaliere": {
        2016: [
            "cnracl_s_ti",
            "cnracl_s_nbi",
            "excep_solidarite",
            "rafp",
            ],
        2017: [
            "cnracl_s_ti",
            "cnracl_s_nbi",
            "excep_solidarite",
            "rafp",
            ],
        2018: [
            "cnracl_s_ti",
            "cnracl_s_nbi",
            "rafp",
            ],
        2019: [
            "cnracl_s_ti",
            "cnracl_s_nbi",
            "rafp",
            ],
        },
    "public_titulaire_territoriale": {
        2016: [
            "cnracl_s_ti",
            "cnracl_s_nbi",
            "excep_solidarite",
            "rafp",
            ],
        2017: [
            "cnracl_s_ti",
            "cnracl_s_nbi",
            "excep_solidarite",
            "rafp",
            ],
        2018: [
            "cnracl_s_ti",
            "cnracl_s_nbi",
            "rafp",
            ],
        2019: [
            "cnracl_s_ti",
            "cnracl_s_nbi",
            "rafp",
            ],
        },
    }


def test_get_baremes_salarie():
    for categorie_salarie, target_by_period in target_by_period_by_categorie_salarie.items():
        for period, target in target_by_period.items():
            assert set(get_baremes_salarie(parameters, categorie_salarie, period, exclude_alsace_moselle = True)) == set(target), f"Error on {categorie_salarie} for {period}"



def get_of2erf(year=None):
    of2erf = dict()
    of2erf["csg"] = "csgim"  # imposable, et "csgdm", déductible
#of2erf["csgd"] = "csgdm"
    of2erf["crds"] = "crdsm"
    of2erf["irpp"] = "zimpom"
    of2erf["ppe"] = "m_ppem"
    of2erf["af"] =  "m_afm"
#af_base
#af_majo
#af_forf
    of2erf["cf"] = "m_cfm"
    of2erf["paje_base"] = "m_pajem"
    of2erf["paje_nais"] = "m_naism"
    of2erf["paje_clca"] = ""
    of2erf["paje_cmg"] = ""
    of2erf["ars"] = "m_arsm"
    of2erf["aeeh"] = "m_aesm" # allocation d'éducation spéciale
    of2erf["asf"] = "m_asfm"
    of2erf["aspa"] = "m_minvm"
    of2erf["aah"] = "m_aahm"
    of2erf["caah"] = "m_caahm"
#of2erf["rmi"] = "m_rmim"
    of2erf["rsa"] = "m_rsam"
    of2erf["rsa_act"] = ""
    of2erf["aefa"] = "m_crmim"
    of2erf["api"] = "m_apim"
    of2erf["logt"] = "logtm"
    of2erf["alf"] = "m_alfm"
    of2erf["als"] = "m_alsm"
    of2erf["apl"] = "m_aplm"
    return of2erf


def get_erf2of():
    of2erf = get_of2erf()
    erf2of = dict((v,k) for k, v in of2erf.items())
    return erf2of



from setuptools import setup, find_packages


with open('README.md') as file:
    long_description = file.read()

setup(
    name = "OpenFisca-France-Data",
    version = "0.22.2",
    description = "OpenFisca-France-Data module to work with French survey data",
    long_description = long_description,
    long_description_content_type="text/markdown",
    author = "OpenFisca Team",
    author_email = "contact@openfisca.fr",
    url = "https://github.com/openfisca/openfisca-france-data",
    license = "http://www.fsf.org/licensing/licenses/agpl-3.0.html",
    keywords = "tax benefit social survey data microsimulation",
    classifiers = [
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering :: Information Analysis",
        ],
    entry_points = {
        'console_scripts': ['build-erfs-fpr=openfisca_france_data.erfs_fpr.input_data_builder:main'],
        },
    python_requires = ">= 3.7",
    install_requires = [
        "click >= 8.0.0, < 9.0.0",
        "matplotlib >= 3.1.1, < 4.0.0",
        "multipledispatch >= 0.6.0, < 1.0.0",
        "openFisca-france >= 113.0.0, < 120.0.0",  # Max 120 because of a bug in OF : https://github.com/openfisca/openfisca-france/issues/1996
        "openFisca-survey-manager >= 0.44.2, < 1.0.0",
        "wquantiles >= 0.3.0, < 1.0.0",  # To compute weighted quantiles
        ],
    extras_require = {
        "test": [
            "autopep8 >= 1.4.0, < 1.5.0",
            "flake8 >= 3.7.0, < 3.8.0",
            "ipython >= 7.5.0, < 8.0.0",
            "mypy >= 0.670, < 1.0.0",
            "pytest >= 4.3.0, < 5.0.0",
            "pytest-cov >= 2.6.0, < 3.0.0",
            "scipy >= 1.2.1, < 2.0.0",
            "toolz >= 0.9.0, < 1.0.0",
            "bumpver >= 2022.1120",
            ],
        },
    packages = find_packages(exclude = ("docs", "tests")),
    )



from typing import Any, Optional

from multipledispatch import dispatch  # type: ignore

from openfisca_core.reforms import Reform  # type: ignore
from openfisca_core.taxbenefitsystems import TaxBenefitSystem  # type: ignore

from openfisca_france_data.erfs_fpr.scenario import ErfsFprSurveyScenario
from openfisca_france_data import france_data_tax_benefit_system


def get_survey_scenario(
        year: int = 2014,
        rebuild_input_data: bool = False,
        tax_benefit_system: Optional[TaxBenefitSystem] = None,
        baseline_tax_benefit_system: Optional[TaxBenefitSystem] = None,
        data: Any = None,  # Plusiers formats possibles.
        reform: Optional[Reform] = None,
        use_marginal_tax_rate: bool = False,
        variation_factor: float = 0.03,
        varying_variable: str = None,
        survey_name: str = "input",
        ) -> ErfsFprSurveyScenario:
    """Helper pour créer un `ErfsFprSurveyScenario`.

    :param year:                        L'année des données de l'enquête.
    :param rebuild_input_data:          Si l'on doit formatter les données (raw) ou pas.
    :param tax_benefit_system:          Le `TaxBenefitSystem` déjà réformé.
    :param baseline_tax_benefit_system: Le `TaxBenefitSystem` au droit courant.
    :param data:                        Les données de l'enquête.
    :param reform:                      Une réforme à appliquer à *france_data_tax_benefit_system*.
    """
    tax_benefit_system = get_tax_benefit_system(
        tax_benefit_system,
        reform,
        )

    baseline_tax_benefit_system = get_baseline_tax_benefit_system(
        baseline_tax_benefit_system,
        )

    if not use_marginal_tax_rate:
        survey_scenario = ErfsFprSurveyScenario.create(
            tax_benefit_system = tax_benefit_system,
            baseline_tax_benefit_system = baseline_tax_benefit_system,
            year = year,
            )
    else:
        assert varying_variable is not None, "You need to specify the varying variable."
        survey_scenario = ErfsFprSurveyScenario.create(
            tax_benefit_system = tax_benefit_system,
            baseline_tax_benefit_system = baseline_tax_benefit_system,
            year = year,
            )
        # taux marginaux !!
        survey_scenario.variation_factor = variation_factor
        survey_scenario.varying_variable = varying_variable

    # S'il n'y a pas de données, on sait où les trouver.
    if data is None:
        input_data_table_by_entity = dict(
            individu = f"individu_{year}",
            menage = f"menage_{year}",
            )

        input_data_table_by_entity_by_period = dict()
        input_data_table_by_entity_by_period[year] = input_data_table_by_entity

        data = dict(
            input_data_table_by_entity_by_period = input_data_table_by_entity_by_period,
            # input_data_survey_prefix = "openfisca_erfs_fpr_data",
            survey = survey_name
            )

    # Les données peuvent venir en différents formats :
    #
    #  * Pandas' DataFrame
    #  * CSV
    #  * HDF5
    #  * Etc.
    #
    #  .. seealso:: :func:`ErfsFprSurveyScenario.init_from_data`
    survey_scenario.init_from_data(
        data = data,
        rebuild_input_data = rebuild_input_data,
        use_marginal_tax_rate = use_marginal_tax_rate,
        )

    return survey_scenario


# Appelé quand *tax_benefit_system* est un :class:`TaxBenefitSystem`
@dispatch(TaxBenefitSystem, object)  # noqa: F811
def get_tax_benefit_system(
        tax_benefit_system: TaxBenefitSystem,
        reform: Optional[None],
        ) -> TaxBenefitSystem:
    return tax_benefit_system


# Appelé quand *tax_benefit_system* est `None`, mais *reform* est une :class:`Reform`
@dispatch(object, Reform)  # noqa: F811
def get_tax_benefit_system(
        tax_benefit_system: None,
        reform: Reform,
        ) -> TaxBenefitSystem:
    return reform(france_data_tax_benefit_system)


# Appelé quand *tax_benefit_system* et *reform* sont `None`
@dispatch(object, object)  # noqa: F811
def get_tax_benefit_system(
        tax_benefit_system: None,
        reform: None,
        ) -> TaxBenefitSystem:
    return france_data_tax_benefit_system


# Appelé quand *tax_benefit_system* est un :class:`TaxBenefitSystem`
@dispatch(TaxBenefitSystem)  # noqa: F811
def get_baseline_tax_benefit_system(
        tax_benefit_system: TaxBenefitSystem,
        ) -> TaxBenefitSystem:
    return tax_benefit_system


# Appelé quand *tax_benefit_system* est `None`
@dispatch(object)  # noqa: F811
def get_baseline_tax_benefit_system(
        tax_benefit_system: None,
        ) -> TaxBenefitSystem:
    return france_data_tax_benefit_system



import collections
from datetime import datetime
import logging
import os

import numpy as np
import pandas as pd

try:
    from ipp_macro_series_parser.config import Config  # type: ignore
except ImportError:
    Config = None

from openfisca_survey_manager.aggregates import AbstractAggregates
from openfisca_france_data import AGGREGATES_DEFAULT_VARS  # type: ignore


log = logging.getLogger(__name__)


class FranceAggregates(AbstractAggregates):
    labels = collections.OrderedDict((
        ('label', "Mesure"),
        ('entity', "Entité"),
        ('reform_amount', "Dépenses\n(millions d'€)"),
        ('reform_beneficiaries', "Bénéficiaires\n(milliers)"),
        ('baseline_amount', "Dépenses initiales\n(millions d'€)"),
        ('baseline_beneficiaries', "Bénéficiaires\ninitiaux\n(milliers)"),
        ('actual_amount', "Dépenses\nréelles\n(millions d'€)"),
        ('actual_beneficiaries', "Bénéficiaires\nréels\n(milliers)"),
        ('amount_absolute_difference', "Diff. absolue\nDépenses\n(millions d'€)"),
        ('beneficiaries_absolute_difference', "Diff absolue\nBénéficiaires\n(milliers)"),
        ('amount_relative_difference', "Diff. relative\nDépenses"),
        ('beneficiaries_relative_difference', "Diff. relative\nBénéficiaires"),
        ))
    aggregate_variables = AGGREGATES_DEFAULT_VARS

    def load_actual_data(self, year = None):
        assert year is not None

        if not Config:
            log.info("No actual data available")
            return

        parser = Config()

        # Cotisations CSG -CRDS
        try:
            directory = os.path.join(
                parser.get('data', 'prelevements_sociaux_directory'),
                'clean',
                )
            csg_crds_amounts = pd.read_csv(
                os.path.join(directory, 'recette_csg_crds.csv'),
                index_col = 0
                ).rename(
                    dict(
                        recette_csg = 'csg',
                        recette_crds = 'crds',
                        )
                    ) / 1e6
            csg_by_type_amounts = pd.read_csv(
                os.path.join(directory, 'recette_csg_by_type.csv'),
                index_col = 0,
                ).drop(
                    ['source']
                    ).astype(float) / 1e6
            assiette_csg_by_type_amounts = pd.read_csv(
                os.path.join(directory, 'assiette_csg_by_type.csv'),
                index_col = 0,
                ) / 1e6
        except Exception:
            assiette_csg_by_type_amounts = None
            csg_by_type_amounts = None
            csg_crds_amounts = None
            pass
        # Prestations sociales
        directory = os.path.join(
            parser.get('data', 'prestations_sociales_directory'),
            'clean',
            )
        amounts_csv = os.path.join(directory, 'historique_depenses.csv')
        beneficiaries_csv = os.path.join(directory, 'historique_beneficiaires.csv')
        prestations_sociales_amounts = pd.read_csv(amounts_csv, index_col = 0)
        prestations_sociales_beneficiaries = pd.read_csv(beneficiaries_csv, index_col = 0)
        # Minimum vieillesses
        minimum_vieillesse_beneficiaries_csv = os.path.join(
            directory, 'historique_beneficiaires_minimum_vieillesse.csv')
        if os.path.exists(minimum_vieillesse_beneficiaries_csv):
            minimum_vieillesse_beneficiaries = pd.read_csv(minimum_vieillesse_beneficiaries_csv, index_col = 0)

        amounts = pd.concat(
            [
                assiette_csg_by_type_amounts,
                csg_by_type_amounts,
                csg_crds_amounts,
                prestations_sociales_amounts,
                ],
            sort = True,
            )
        beneficiaries = pd.concat(
            [minimum_vieillesse_beneficiaries, prestations_sociales_beneficiaries],
            sort = True,
            )

        return pd.DataFrame(data = {
            "actual_amount": amounts[str(year)],
            "actual_beneficiaries": beneficiaries[str(year)],
            })



from openfisca_france_data.model.base import * # noqa  analysis:ignore


class menage_ordinaire(Variable):
    value_type = int
    is_period_size_independent = True
    default_value = True
    entity = Menage
    definition_period = YEAR

class menage_ordinaire_individus(Variable):
    value_type = int
    is_period_size_independent = True
    entity = Individu
    label = "L'individu est dans un ménage du champ ménage"
    definition_period = YEAR

    def formula(individu, period):
        return individu.menage('menage_ordinaire', period)


class menage_ordinaire_familles(Variable):
    value_type = int
    is_period_size_independent = True
    entity = Famille
    label = "Le premier parent de la famille est dans un ménage du champ ménage"
    definition_period = YEAR

    def formula(famille, period):
        return famille.demandeur('menage_ordinaire_individus', period)


class menage_ordinaire_foyers_fiscaux(Variable):
    value_type = int
    is_period_size_independent = True
    entity = FoyerFiscal
    label = "Le premier déclarant du foyer est dans un ménage du champ ménage"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        return foyer_fiscal.declarant_principal('menage_ordinaire_individus', period)


class weight_familles(Variable):
    is_period_size_independent = True
    value_type = float
    entity = Famille
    label = "Poids de la famille"
    definition_period = YEAR

    def formula(famille, period):
        return famille.demandeur('weight_individus', period)


class weight_foyers(Variable):
    is_period_size_independent = True
    value_type = float
    entity = FoyerFiscal
    label = "Poids du foyer fiscal"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        return foyer_fiscal.declarant_principal('weight_individus', period)


class weight_menages(Variable):
    is_period_size_independent = True
    value_type = float
    entity = Menage
    label = "Poids du ménage"
    definition_period = YEAR

    def formula(menage, period):
        return menage.personne_de_reference('weight_individus', period)



class wprm(Variable):
    default_value = 1.0
    is_period_size_independent = True
    value_type = float
    entity = Menage
    label = "Effectifs"
    definition_period = YEAR


class wprm_init(Variable):
    is_period_size_independent = True
    value_type = float
    entity = Menage
    label = "Effectifs"
    definition_period = YEAR


class weight_individus(Variable):
    is_period_size_independent = True
    value_type = float
    entity = Individu
    label = "Poids de l'individu"
    definition_period = YEAR

    def formula(individu, period):
        return individu.menage('wprm', period)
